@@ -88,6 +88,7 @@
 //! ```
 
 use std::fmt;
+use std::cmp;
 
 use Expr::*;
 
@@ -108,6 +109,85 @@ pub enum Expr {
     Jok(Box<Expr>),
 }
 
+impl cmp::PartialOrd for Expr {
+    fn partial_cmp(&self, other: &Self) -> Option<cmp::Ordering> {
+        fn seq_jok(x: &Expr, y: &Expr, v: &Expr, v2: &Expr) -> bool {
+            match (x, y) {
+                (Seq(x, x2), Jok(y)) =>
+                    if &**x == v && &**y == v {
+                        if &**x2 == v2 {true}
+                        else {false}
+                    } else {false},
+                _ => false
+            }
+        }
+
+        use cmp::Ordering::*;
+
+        match (self, other) {
+            (x, y) if x == y => Some(Equal),
+            (_0, _1) => Some(Less),
+            (_1, _0) => Some(Greater),
+            (Jok(x), Jok(y)) => x.partial_cmp(y),
+            (_0, Jok(y)) if **y == _0 => Some(Less),
+            (Jok(x), _0) if **x == _0 => Some(Greater),
+            (_1, Jok(y)) if **y == _1 => Some(Greater),
+            (Jok(x), _1) if **x == _1 => Some(Less),
+            (x, y) if seq_jok(x, y, &_0, &joker(_0)) => Some(Less),
+            (x, y) if seq_jok(y, x, &_0, &joker(_0)) => Some(Greater),
+            (x, y) if seq_jok(x, y, &_1, &joker(_1)) => Some(Greater),
+            (x, y) if seq_jok(y, x, &_1, &joker(_1)) => Some(Less),
+            (x, y) if seq_jok(x, y, &_0, &_1) => Some(Greater),
+            (x, y) if seq_jok(y, x, &_0, &_1) => Some(Less),
+            (x, y) if seq_jok(x, y, &_1, &_0) => Some(Less),
+            (x, y) if seq_jok(y, x, &_1, &_0) => Some(Greater),
+            (_0, Seq(x, x2)) if **x == _1 && **x2 == _0 => Some(Less),
+            (Seq(x, x2), _0) if **x == _1 && **x2 == _0 => Some(Greater),
+            (_1, Seq(x, x2)) if **x == _0 && **x2 == _1 => Some(Greater),
+            (Seq(x, x2), _1) if **x == _0 && **x2 == _1 => Some(Less),
+            (Jok(x), _1) if **x == _0 => Some(Less),
+            (_1, Jok(x)) if **x == _0 => Some(Greater),
+            (_0, Jok(x)) if **x == _1 => Some(Less),
+            (Jok(x), _0) if **x == _1 => Some(Greater),
+            (_0, Seq(x, x2)) if **x == _0 && **x2 == joker(_0) => Some(Less),
+            (Seq(x, x2), _0) if **x == _0 && **x2 == joker(_0) => Some(Greater),
+            (_1, Seq(x, x2)) if **x == _1 && **x2 == joker(_1) => Some(Greater),
+            (Seq(x, x2), _1) if **x == _1 && **x2 == joker(_1) => Some(Less),
+            (Seq(x, x2), Jok(y)) if **x == _0 && **x2 == joker(_0) && **y == _1 => Some(Less),
+            (Jok(y), Seq(x, x2)) if **x == _0 && **x2 == joker(_0) && **y == _1 => Some(Greater),
+            (Seq(x, x2), _1) if **x == _0 && **x2 == joker(_0) => Some(Less),
+            (_1, Seq(x, x2)) if **x == _0 && **x2 == joker(_0) => Some(Greater),
+            (_0, Seq(x, x2)) if **x == _1 && **x2 == joker(_1) => Some(Less),
+            (Seq(x, x2), _0) if **x == _1 && **x2 == joker(_1) => Some(Greater),
+            (_0, Seq(x, x2)) if **x == _0 && **x2 == _1 => Some(Less),
+            (Seq(x, x2), _0) if **x == _0 && **x2 == _1 => Some(Greater),
+            (_1, Seq(x, x2)) if **x == _1 && **x2 == _0 => Some(Greater),
+            (Seq(x, x2), _1) if **x == _1 && **x2 == _0 => Some(Less),
+            (Seq(x, x2), Seq(y, y2))
+                if **x == _0 && **x2 == joker(_0) &&
+                   **y == _1 && **y2 == joker(_1) => Some(Less),
+            (Seq(x, x2), Seq(y, y2))
+                if **x == _1 && **x2 == joker(_1) &&
+                   **y == _0 && **y2 == joker(_0) => Some(Greater),
+            (Seq(x, x2), Seq(y, y2))
+                if **x == _0 && **x2 == joker(_0) &&
+                   **y == _0 && **y2 == _1 => Some(Less),
+            (Seq(x, x2), Seq(y, y2))
+                if **x == _0 && **x2 == _1 &&
+                   **y == _0 && **y2 == joker(_0) => Some(Greater),
+            (Seq(x, x2), Seq(y, y2))
+                if **x == _1 && **x2 == _0 &&
+                   **y == _1 && **y2 == joker(_1) => Some(Less),
+            (Seq(x, x2), Seq(y, y2))
+                if **x == _1 && **x2 == joker(_1) &&
+                   **y == _1 && **y2 == _0 => Some(Greater),
+            (Jok(x), Seq(y, y2)) if **x == _0 && **y == _1 && **y2 == joker(_1) => Some(Less),
+            (Seq(y, y2), Jok(x)) if **x == _0 && **y == _1 && **y2 == joker(_1) => Some(Greater),
+            _ => None,
+        }
+    }
+}
+
 impl fmt::Display for Expr {
     fn fmt(&self, w: &mut fmt::Formatter<'_>) -> Result<(), std::fmt::Error> {
         match self {
@@ -316,6 +396,22 @@ impl Expr {
     pub fn contracting(&self, closed: bool) -> bool {
         seq(self.clone(), self.clone()).eval(closed) == self.eval(closed)
     }
+
+    /// Returns `true` if expression contains a joker.
+    pub fn contains_joker(&self) -> bool {
+        match self {
+            _1 | _0 => false,
+            Jok(_) => true,
+            Seq(a, b) | Sel(a, b) => a.contains_joker() || b.contains_joker(),
+            Not(a) => a.contains_joker(),
+        }
+    }
+
+    /// Returns `true` if expression is "authentic" in sense of Heidegger.
+    ///
+    /// This is defined as the evaluated expression in Closed Joker Calculus
+    /// does not contain a joker.
+    pub fn authentic(&self) -> bool {!self.eval_closed().contains_joker()}
 }
 
 /// Platonism (terminal expression).
@@ -352,18 +448,16 @@ pub fn sel<T: Into<Expr>, U: Into<Expr>>(a: T, b: U) -> Expr {
 macro_rules! jc (
     ( 0 ) => {platonism()};
     ( 1 ) => {seshatism()};
+    ( $x:tt , $($y:tt)+ ) => {sel(jc!($x), jc!($($y)+))};
+    ( $x0:tt $x1:tt , $($y:tt)+ ) => {sel(jc!($x0 $x1), jc!($($y)+))};
+    ( $x0:tt $x1:tt $x2:tt , $($y:tt)+ ) => {sel(jc!($x0 $x1 $x2), jc!($($y)+))};
+    ( $x0:tt $x1:tt $x2:tt $x3:tt , $($y:tt)+ ) => {sel(jc!($x0 $x1 $x2 $x3), jc!($($y)+))};
+    ( ( $($x:tt)+ ) $($y:tt)+ ) => {seq(jc!($($x)+), jc!($($y)+))};
+    ( ( $($x:tt)+ ) ) => {jc!($($x)+)};
     ( ? $($x:tt)+ ) => {joker(jc!($($x)+))};
     ( ! $($x:tt)+ ) => {not(jc!($($x)+))};
     ( 0 $($x:tt)+ ) => {platonic(jc!($($x)+))};
     ( 1 $($x:tt)+ ) => {seshatic(jc!($($x)+))};
-    ( ( 0 , $($x:tt)+ ) ) => {sel(platonism(), jc!($($x)+))};
-    ( ( 1 , $($x:tt)+ ) ) => {sel(seshatism(), jc!($($x)+))};
-    ( ( 0 $x:tt , $($y:tt)+ ) ) => {sel(platonic(jc!($x)), jc!($($y)+))};
-    ( ( 1 $x:tt , $($y:tt)+ ) ) => {sel(seshatic(jc!($x)), jc!($($y)+))};
-    ( ( 0 ? $x:tt , $($y:tt)+ ) ) => {sel(platonic(joker(jc!($x))), jc!($($y)+))};
-    ( ( 1 ? $x:tt , $($y:tt)+ ) ) => {sel(seshatic(joker(jc!($x))), jc!($($y)+))};
-    ( ( 0 ! $x:tt , $($y:tt)+ ) ) => {sel(platonic(not(jc!($x))), jc!($($y)+))};
-    ( ( 1 ! $x:tt , $($y:tt)+ ) ) => {sel(seshatic(not(jc!($x))), jc!($($y)+))};
 );
 
 #[cfg(test)]
@@ -568,4 +662,67 @@ mod tests {
         assert_eq!(a.swap_open(), joker(joker(platonism())));
         assert_eq!(a.swap_closed(), seshatism());
     }
+
+    #[test]
+    fn test_macro() {
+        let a = jc!(0);
+        assert_eq!(a, platonism());
+
+        let a = jc!(1);
+        assert_eq!(a, seshatism());
+
+        let a = jc!(0 1);
+        assert_eq!(a, platonic(seshatism()));
+
+        let a = jc!(0, 1);
+        assert_eq!(a, sel(platonism(), seshatism()));
+
+        let a = jc!((0, 1));
+        assert_eq!(a, sel(platonism(), seshatism()));
+
+        let a = jc!(?0);
+        assert_eq!(a, joker(platonism()));
+
+        let a = jc!(0 ?0);
+        assert_eq!(a, platonic(joker(platonism())));
+
+        let a = jc!(?(0 1));
+        assert_eq!(a, joker(platonic(seshatism())));
+
+        let a = jc!(?(0, 1));
+        assert_eq!(a, joker(sel(platonism(), seshatism())));
+
+        let a = jc!(?0, !0);
+        assert_eq!(a, sel(joker(platonism()), not(platonism())));
+
+        let a = jc!(0 ?1, 0);
+        assert_eq!(a, sel(platonic(joker(seshatism())), platonism()));
+
+        let a = jc!(0 !1, 0);
+        assert_eq!(a, sel(platonic(not(seshatism())), platonism()));
+
+        let a = jc!(0 (1, 0));
+        assert_eq!(a, platonic(sel(seshatism(), platonism())));
+
+        let a = jc!((0, 1) (1, 0));
+        assert_eq!(a, seq(sel(platonism(), seshatism()), sel(seshatism(), platonism())));
+    }
+
+    #[test]
+    fn test_order() {
+        fn check_order(order: &[Expr]) {
+            for i in 0..order.len() {
+                for j in i+1..order.len() {
+                    assert!(order[i] < order[j], "{} < {}", order[i], order[j]);
+                    assert!(order[j] > order[i], "{} > {}", order[j], order[i]);
+                }
+            }
+        }
+
+        check_order(&[jc!(0), jc!(0 ?0), jc!(?0), jc!(?1), jc!(1 ?1), jc!(1)]);
+        check_order(&[jc!(0), jc!(0 ?0), jc!(?0), jc!(0 1)]);
+        check_order(&[jc!(1 0), jc!(?1), jc!(1 ?1), jc!(1)]);
+        check_order(&[jc!(0), jc!(1 0)]);
+        check_order(&[jc!(0 1), jc!(1)]);
+    }
 }
